<?php

namespace Drupal\views_kanban\Controller;

use Drupal\Core\Url;
use Drupal\views\Views;
use Drupal\workflows\State;
use Drupal\user\Entity\User;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class Kanban Controller.
 */
class KanbanController extends ControllerBase {

  /**
   * Update state.
   *
   * @param string $view_id
   *   View id.
   * @param string $display_id
   *   Display id.
   * @param int $entity_id
   *   Entity id.
   * @param string $state_value
   *   State value.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Return json.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function updateState($view_id, $display_id, $entity_id = 0, $state_value = '') {
    $message = NULL;
    $data = [
      'success' => FALSE,
      'message' => $message,
    ];
    if (!$state_value && !is_numeric($entity_id)) {
      return new JsonResponse($data);
    }
    $view = Views::getView($view_id);
    $handler = $view->getHandler($display_id, 'filter', 'type');
    $entity_type = !empty($handler['entity_type']) ? $handler['entity_type'] : 'user';
    $style_plugin = $view->display_handler->getPlugin('style');
    $status_field = $style_plugin->options["status_field"];
    $entity = \Drupal::entityTypeManager()
      ->getStorage($entity_type)
      ->load($entity_id);
    $message = $this->getHistoryMessage($entity, $status_field, $state_value);
    $statusName = $this->getStatusName($entity, $status_field, $state_value);

    if (!array_key_exists($state_value, $this->getAllowedValues($entity, $status_field))) {
      $data['message'] = $this->t(
        'New state @state is not a valid', ['@state' => $state_value]
      );
      return new JsonResponse($data);
    }
    $extractStatus = explode(':', $status_field);
    if (!empty($extractStatus[1])) {
      $status_field = $extractStatus[0];
    }
    // Save new status.
    $entity->set($status_field, $state_value);

    // Save history.
    if (!empty($historyFieldName = $style_plugin->options["history_field"])) {
      $historyField = $entity->get($style_plugin->options["history_field"]);
      $historyType = $historyField->getFieldDefinition()->getType();
      $historyValue = \Drupal::service('date.formatter')
        ->format(strtotime('now')) . ' ' . $message;
      if ($historyType == 'double_field') {
        $historyValue = [
          'first' => date('Y-m-d\TH:i:s'),
          'second' => $message,
        ];
      }
      $entity->$historyFieldName->appendItem($historyValue);
    }

    $entity->save();

    // Send email to assignor.
    if (!empty($style_plugin->options["send_email"])) {
      $user = User::load(\Drupal::currentUser()->id());
      $email = $user->getEmail();
      $name = $user->getDisplayName();
      $author_initial = implode('', array_map(function ($v) {
        return $v[0];
      },
        explode(' ', $name)));
      $assignValues[$entity->getOwnerID()] = [$entity->getOwnerID()];
      if (!empty($style_plugin->options["assign_field"])) {
        $assignors = $entity->get($style_plugin->options["assign_field"])
          ->getValue();
        foreach ($assignors as $assignor) {
          $assignValues[$assignor['target_id']] = $assignor['target_id'];
        }
      }
      $mailManager = \Drupal::service('plugin.manager.mail');
      $key = $view_id . '-' . $view->current_display;
      $link = Url::fromRoute(implode('.', [
        'view',
        $view_id,
        $view->current_display,
      ]))->setOption('absolute', TRUE)
        ->setOption('query', ['kanbanTicket' => $entity_id])
        ->toString();
      foreach ($assignValues as $uid) {
        $assignor = User::load($entity->getOwnerID());
        $to = $assignor->getEmail();

        // Set up email template.
        $body_data = [
          '#theme' => 'views_email_kanban',
          '#message' => $message,
          '#author_initial' => strtoupper($author_initial),
          '#type' => $statusName['new'],
          '#author_name' => $name,
          '#title' => $this->t("Change status") . " - " . $entity->label(),
          '#assignator' => $assignor->getDisplayName(),
          '#btn_text' => $this->t('View'),
          '#link' => $link,
        ];
        $message = [
          'id' => $key,
          'headers' => [
            'Content-type' => 'text/html; charset=UTF-8; format=flowed; delsp=yes',
            'Reply-to' => $name . '<' . $email . '>',
            'Content-Transfer-Encoding' => '8Bit',
            'MIME-Version' => '1.0',
          ],
          'subject' => $view->getTitle() . ' ' . \Drupal::config('system.site')
            ->get('name'),
          'to' => $to,
          'body' => \Drupal::service('renderer')->render($body_data),
        ];
        $mailManager->getInstance(['module' => 'views_kanban', 'key' => $key])
          ->mail($message);
      }
    }
    $data = [
      'success' => TRUE,
      'message' => $message,
    ];
    return new JsonResponse($data);
  }

  /**
   * Get the value allowed in the state field.
   *
   * @param object $entity
   *   Entity.
   * @param string $fieldName
   *   Field name.
   *
   * @return array
   *   Allow values.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getAllowedValues($entity, $fieldName) {
    $extractStatus = explode(':', $fieldName);
    if (!empty($extractStatus[1])) {
      $fieldName = $extractStatus[0];
      $workflow_id = $extractStatus[1];
      $workflow = \Drupal::entityTypeManager()
        ->getStorage('workflow')
        ->load($workflow_id);
      return array_map([
        State::class,
        'labelCallback',
      ], $workflow->getTypePlugin()->getStates());
    }
    $statusFieldDefinition = $entity->get($fieldName)->getFieldDefinition();
    $statusFieldValues = $statusFieldDefinition->getSettings();
    $allowed_values = [];
    if (!empty($statusFieldValues["allowed_values"])) {
      $allowed_values = $statusFieldValues["allowed_values"];
    }
    if (!empty($statusFieldValues["target_type"])) {
      $vid = current($statusFieldValues["handler_settings"]["target_bundles"]);
      $loadTermStatus = \Drupal::entityTypeManager()
        ->getStorage('taxonomy_term')
        ->loadTree($vid);
      foreach ($loadTermStatus as $term) {
        $allowed_values[$term->tid] = $term->name;
      }
    }
    return $allowed_values;
  }

  /**
   * Get message for log.
   *
   * @param object $entity
   *   Entity.
   * @param string $status_field
   *   Status field.
   * @param string $newStatus
   *   New status.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   Array text history status.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getHistoryMessage($entity, $status_field, $newStatus) {
    $statusName = $this->getStatusName($entity, $status_field, $newStatus);
    $account = \Drupal::currentUser();
    return $this->t('@user change from @old to @new', [
      '@user' => $account->getDisplayName(),
      '@old' => $statusName['old'],
      '@new' => $statusName['new'],
    ]
    );
  }

  /**
   * Get status name.
   */
  protected function getStatusName($entity, $status_field, $newStatus) {
    $statusList = $this->getAllowedValues($entity, $status_field);
    $extractStatus = explode(':', $status_field);
    if (!empty($extractStatus[1])) {
      $status_field = $extractStatus[0];
    }
    $currentStatus = $entity->get($status_field)->getString();
    if (is_array($currentStatus)) {
      $currentStatus = current($currentStatus);
    }
    if (!empty($statusList[$currentStatus])) {
      $currentStatus = $statusList[$currentStatus];
    }
    if (!empty($statusList[$newStatus])) {
      $newStatus = $statusList[$newStatus];
    }
    return [
      'old' => $currentStatus,
      'new' => $newStatus,
    ];
  }

}
